import argparse
import logging
import os
from collections import namedtuple
from subprocess import run, PIPE, STDOUT, CompletedProcess

from nldcsc.loggers.app_logger import AppLogger

logging.setLoggerClass(AppLogger)


class FlaskSqlMigrate(object):
    def __init__(self, cwd: str = None, app_ref: str = None):
        """
        app_ref:    The Flask application or factory function to load, in
                    the form 'module:name'. Module can be a dotted import
                    or file path. Name is not required if it is 'app',
                    'application', 'create_app', or 'make_app', and can be
                    'name(args)' to pass arguments.
        """
        self.logger = logging.getLogger(__name__)

        self.current_dir = (
            cwd if cwd is not None else os.path.dirname(os.path.realpath(__file__))
        )

        self._commands = namedtuple("commands", "INIT MIGRATE UPDATE STAMP")(1, 2, 3, 4)

        self.app_ref = app_ref

        if self.app_ref is not None:
            if "/" in self.app_ref:
                if os.path.exists(self.app_ref):
                    if ":" in self.app_ref:
                        self.current_dir = os.path.dirname(self.app_ref.split(":")[0])
                    else:
                        self.current_dir = os.path.dirname(self.app_ref)
                        if self.current_dir == "":
                            raise ValueError(
                                f"Could not parse parent dir from the given filename, please check and try again"
                            )
                else:
                    raise FileNotFoundError(
                        f"Cannot find the file you are referring to: {self.app_ref}, please check and try again!"
                    )
            else:
                if cwd is None:
                    raise AttributeError(
                        f"If you are providing a dotted module string, you need to provide a value for the "
                        f"'cwd' variable"
                    )
                else:
                    if not os.path.exists(self.current_dir):
                        raise FileNotFoundError(
                            f"Cannot find the directory you are referring to: {self.current_dir}, "
                            f"please check and try again!"
                        )

    @property
    def commands(self) -> namedtuple:
        return self._commands

    def db_init(self) -> None:
        res = self.__cli_runner(self.commands.INIT)
        self.__parse_command_output(res)

    def db_migrate(self) -> None:
        res = self.__cli_runner(self.commands.MIGRATE)
        self.__parse_command_output(res)

    def db_update(self) -> None:
        res = self.__cli_runner(self.commands.UPDATE)
        self.__parse_command_output(res)

    def db_stamp(self) -> None:
        res = self.__cli_runner(self.commands.STAMP)
        self.__parse_command_output(res)

    def __parse_command_output(self, cmd_output: CompletedProcess) -> None:
        if cmd_output.returncode != 0:
            self.logger.error(cmd_output.stdout.split("\n")[0])
        else:
            output_list = cmd_output.stdout.split("\n")

            for m in output_list:
                if m != "":
                    self.logger.info(m)

    def __cli_runner(self, command: int) -> CompletedProcess:
        command_mapping = {
            1: f"flask{f' --app {self.app_ref}' if self.app_ref is not None else ''} db init",
            2: f"flask{f' --app {self.app_ref}' if self.app_ref is not None else ''} db migrate",
            3: f"flask{f' --app {self.app_ref}' if self.app_ref is not None else ''} db upgrade",
            4: f"flask{f' --app {self.app_ref}' if self.app_ref is not None else ''} db stamp head",
        }
        try:
            result = run(
                command_mapping[command],  # nosec
                stdout=PIPE,
                stderr=STDOUT,
                universal_newlines=True,
                shell=True,
                cwd=self.current_dir,
            )
            return result
        except KeyError:  # pragma: no cover
            self.logger.error(f"Unknown command number received....")

    def __repr__(self) -> str:
        return f"<< FlaskSqlMigrate >>"


if __name__ == "__main__":
    argparser = argparse.ArgumentParser(
        description="migrate/update the database schema"
    )

    argparser.add_argument(
        "-i", action="store_true", help="Setup new migration directory"
    )
    argparser.add_argument("-m", action="store_true", help="Migrate the database")
    argparser.add_argument("-u", action="store_true", help="Update the database")
    argparser.add_argument(
        "-s",
        action="store_true",
        help="Stamp the head of the database to current state",
    )
    argparser.add_argument(
        "-a",
        "--app",
        type=str,
        help="The Flask application or factory function "
        "to load, in the form 'module:name'. Module can be a "
        "dotted import or file path. Name is not required if "
        "it is 'app', 'application', 'create_app', or 'make_app', "
        "and can be 'name(args)' to pass arguments.",
    )
    args = argparser.parse_args()

    if args.app:
        fsm = FlaskSqlMigrate(app_ref=args.app)
    else:
        fsm = FlaskSqlMigrate()

    if args.i:
        fsm.db_init()

    if args.m:
        fsm.db_migrate()

    if args.u:
        fsm.db_update()

    if args.s:
        fsm.db_stamp()



import calendar
from datetime import datetime


ISOFORMAT = "%Y-%m-%dT%H:%M:%SZ"


def timestringTOtimestamp(timestring):
    """
    Method to convert a given date time string into a timestamp. Timestring is matched to the date_time_formats
    'date time string formats' list. If matched will return a timestamp integer; will return False otherwise.

    :param timestring: date time string
    :type timestring: str
    :return: unix timestamp
    :rtype: int
    """

    date_time_formats = [
        "%d-%m-%Y",
        "%d-%m-%Y %H:%M",
        "%Y-%m-%d %H:%M:%S",
        "%H:%M:%S %d-%m-%Y",
        "%Y-%m-%dT%H:%M:%S",
        "%Y-%m-%dT%H:%M:%SZ",
        "%Y-%m-%dT%H:%M:%S.%f",
        "%Y-%m-%dT%H:%M:%S,%f",
        "%Y-%m-%dT%H:%M:%S.%fZ",
        "%Y-%m-%dT%H:%M:%S,%fZ",
        "%Y-%m-%dT%H:%M:%S.%fZ%z",
        "%Y-%m-%dT%H:%M:%S,%fZ%z",
        "%Y-%m-%d %H:%M:%S.%f%z",
    ]
    match = False

    # try to match string formats to given string
    for each in date_time_formats:
        try:
            match = datetime.strptime(timestring, each)
        except ValueError:
            continue

    if match:
        match = calendar.timegm(match.utctimetuple())

    return match


def datetimeTOtimestamp(date_time_object):
    return calendar.timegm(date_time_object.utctimetuple())


def dateTOtimestamp(date_time_object):
    return calendar.timegm(date_time_object.timetuple())


def timestampTOdatetime(timestamp):
    """
    Method that will take the provided timestamp and converts it into a date time object

    :param timestamp: unix timestamp
    :type timestamp: int
    :return: date time object
    :rtype: datetime
    """
    value = datetime.utcfromtimestamp(timestamp)

    return value


def timestampTOdatestring(timestamp):
    """
    Method that will take the provided timestamp and converts it into a date time string

    :param timestamp: unix timestamp
    :type timestamp: int
    :return: date time object
    :rtype: datetime.datetime (format: '%d-%m-%Y')
    """
    value = datetime.utcfromtimestamp(timestamp)

    return value.strftime("%d-%m-%Y")


def timestampTOdatetimestring(timestamp, vis=False):
    """
    Method that will take the provided timestamp and converts it into a RFC3339 date time string

    :param timestamp: unix timestamp
    :type timestamp: int
    :return: date time object
    :rtype: datetime.datetime (format: '%d-%m-%YT%H:%M:%SZ')
    """
    value = datetime.utcfromtimestamp(timestamp)

    if not vis:
        return value.strftime("%Y-%m-%dT%H:%M:%SZ")
    else:
        return value.strftime("%Y-%m-%d %H:%M:%S")


def timestampTOcalendarattrs(timestamp):
    value = datetime.utcfromtimestamp(timestamp)

    day = value.strftime("%d")
    month = value.strftime("%B")
    weekday = value.strftime("%A")

    return {"day": day, "month": month, "weekday": weekday}


def timestringTOdatetimestring(timestring):
    timestamp = timestringTOtimestamp(timestring=timestring)

    if timestamp is False:
        return False

    return timestampTOdatetimestring(timestamp=timestamp)



#!/usr/bin/env python3
import codecs
import os
import re
from itertools import chain

from setuptools import setup, find_packages

__NAME__ = "nldcsc"

# -*- Distribution Meta -*-

re_meta = re.compile(r"__(\w+?)__\s*=\s*(.*)")
re_doc = re.compile(r'^"""(.+?)"""')


def _add_default(m):
    attr_name, attr_value = m.groups()
    return ((attr_name, attr_value.strip("\"'")),)


def _add_doc(m):
    return (("doc", m.groups()[0]),)


def parse_dist_meta():
    """Extract metadata information from ``$dist/__init__.py``."""
    pats = {re_meta: _add_default, re_doc: _add_doc}
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, __NAME__, "__init__.py")) as meta_fh:
        distmeta = {}
        for line in meta_fh:
            if line.strip() == "# -eof meta-":
                break
            for pattern, handler in pats.items():
                m = pattern.match(line.strip())
                if m:
                    distmeta.update(handler(m))
        return distmeta


# -*- Extras -*-

MODULES = {
    "auth",
    "datatables",
    "flask_managers",
    "flask_middleware",
    "flask_plugins",
    "http_apis",
    "loggers",
    "sql_migrations",
    "sso",
}

# -*- Requirements -*-


def _strip_comments(l):
    return l.split("#", 1)[0].strip()


def _pip_requirement(req):
    if req.startswith("-r "):
        _, path = req.split()
        return reqs(*path.split("/"))
    return [req]


def _reqs(*f):
    if len(f) == 2:
        if os.getcwd().endswith("modules") and f[0] == "modules":
            f = [f[1]]
        if not os.getcwd().endswith("modules") and f[0] == ".":
            f = ("modules", f[1])
    return [
        _pip_requirement(r)
        for r in (
            _strip_comments(l)
            for l in open(os.path.join(os.getcwd(), "requirements", *f)).readlines()
        )
        if r
    ]


def reqs(*f):
    """Parse requirement file.

    Example:
        reqs('default.txt')          # requirements/default.txt
        reqs('modules', 'redis.txt')  # requirements/modules/redis.txt
        reqs('.', 'loggers.txt')  # requirements/modules/loggers.txt -> this is a reference in a requirements file
        like -r ./loggers.txt
    Returns:
        List[str]: list of requirements specified in the file.
    """
    return [req for subreq in _reqs(*f) for req in subreq]


def extras(*p):
    """Parse requirement in the requirements/modules/ directory."""
    return reqs("modules", *p)


def install_requires():
    """Get list of requirements required for installation."""
    return reqs("default.txt")


def extras_require():
    """Get map of all extra requirements."""
    module_requirements = {x: extras(x + ".txt") for x in MODULES}

    # add an 'all' value to install all requirements for all modules
    module_requirements["all"] = list(set(chain(*module_requirements.values())))

    return module_requirements


# -*- Long Description -*-


def long_description():
    try:
        return codecs.open("README.md", "r", "utf-8").read()
    except OSError:
        return "Long description error: Missing README.md file"


meta = parse_dist_meta()

setup(
    name=__NAME__,
    packages=find_packages(exclude=["tests", "test_data"]),
    version=meta["version"],
    description="Package with general devops code",
    long_description=long_description(),
    long_description_content_type="text/markdown",
    author=meta["author"],
    author_email="NLDCSC@invalid.com",
    url="https://github.com/NLDCSC/nldcsc",
    license="GNU General Public License v3.0",
    platforms=["any"],
    install_requires=install_requires(),
    extras_require=extras_require(),
    python_requires=">=3.10",
    include_package_data=True,
    project_urls={
        "Code": "https://github.com/NLDCSC/nldcsc",
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Operating System :: OS Independent",
    ],
)
